local api = require("gemini.api")
local M = {}

-- State for managing current suggestion and debounce timer
local current_suggestion = {
    text = nil,
    start_col = nil,
    namespace_id = vim.api.nvim_create_namespace('gemini_suggestion'),
    timer = nil
}

-- Helper function to clear suggestion
local function clear_suggestion()
    if current_suggestion.timer then
        vim.fn.timer_stop(current_suggestion.timer)
        current_suggestion.timer = nil
    end
    
    if current_suggestion.text then
        vim.api.nvim_buf_clear_namespace(0, current_suggestion.namespace_id, 0, -1)
        current_suggestion.text = nil
        current_suggestion.start_col = nil
    end
end

-- Helper function to show suggestion
local function show_suggestion(suggestion, start_col)
    clear_suggestion()
    
    -- Only use first line of suggestion
    suggestion = suggestion:match("^[^\n]*")
    if suggestion == "" then return end
    
    local line = vim.api.nvim_win_get_cursor(0)[1] - 1
    local line_text = vim.api.nvim_buf_get_lines(0, line, line + 1, true)[1]
    
    -- Validate column position
    if start_col > #line_text then
        return
    end
    
    current_suggestion.text = suggestion
    current_suggestion.start_col = start_col
    
    -- Make the text virtual (doesn't affect actual buffer content)
    vim.api.nvim_buf_set_extmark(0, current_suggestion.namespace_id, line, start_col, {
        virt_text = {{suggestion, 'GeminiSuggestion'}},
        virt_text_pos = 'inline',
        virt_text_hide = true,
    })
end

-- Function to accept current suggestion
function M.accept_suggestion()
    if not (current_suggestion.text and current_suggestion.start_col) then
        return false
    end

    local line = vim.api.nvim_win_get_cursor(0)[1] - 1
    local line_text = vim.api.nvim_buf_get_lines(0, line, line + 1, true)[1]
    
    -- Validate column position
    if current_suggestion.start_col > #line_text then
        clear_suggestion()
        return false
    end
    
    vim.api.nvim_buf_set_text(
        0,
        line,
        current_suggestion.start_col,
        line,
        current_suggestion.start_col,
        {current_suggestion.text}
    )
    clear_suggestion()
    return true
end

function M.trigger_completion()
    -- Clear any existing timer
    if current_suggestion.timer then
        vim.fn.timer_stop(current_suggestion.timer)
    end
    
    -- Start a new timer for debouncing
    current_suggestion.timer = vim.fn.timer_start(100, function()
        local cursor = vim.api.nvim_win_get_cursor(0)
        local line = cursor[1]
        local col = cursor[2]
        
        -- Get entire buffer content
        local lines = vim.api.nvim_buf_get_lines(0, 0, -1, false)
        local current_line = lines[line]
        
        -- Don't trigger completion if line is empty or cursor is at the start
        if col == 0 or current_line:match("^%s*$") then
            clear_suggestion()
            return
        end
        
        -- Split into before and after cursor
        local before_lines = {}
        
        -- Copy lines before current line
        for i = 1, line - 1 do
            table.insert(before_lines, lines[i])
        end
        
        -- Add current line (complete line for context)
        table.insert(before_lines, current_line)
        
        -- Combine all lines
        local prefix = table.concat(before_lines, "\n")
        
        -- Construct prompt for Gemini
        local prompt = string.format(
            "Complete this code. Only provide the completion, no explanation:\n%s",
            prefix
        )
        
        -- Get completion from Gemini
        api.get_response(prompt, nil, function(response, error)
            if error then return end
            
            if type(response) == "string" then
                -- Get first non-empty line as suggestion
                local suggestion = response:match("^%s*(.-)%s*$")
                if suggestion and #suggestion > 0 then
                    vim.schedule(function()
                        -- Check if cursor position is still the same
                        local new_cursor = vim.api.nvim_win_get_cursor(0)
                        if new_cursor[1] == line and new_cursor[2] == col then
                            show_suggestion(suggestion, col)
                        end
                    end)
                end
            end
        end)
    end)
end

-- Setup function to create highlight group and keymaps
function M.setup()
    -- Create highlight group for suggestions
    vim.api.nvim_set_hl(0, 'GeminiSuggestion', {
        fg = '#666666',
        italic = true,
    })
    
    -- Map Tab to accept suggestion or behave normally
    vim.keymap.set('i', '<Tab>', function()
        if not M.accept_suggestion() then
            -- If no suggestion to accept, send regular Tab key
            vim.api.nvim_feedkeys(vim.api.nvim_replace_termcodes('<Tab>', true, true, true), 'n', true)
        end
    end, { expr = false, noremap = true })
    
    -- Set up autocommand for real-time completion
    vim.api.nvim_create_autocmd("TextChangedI", {
        callback = function()
            M.trigger_completion()
        end
    })
    
    -- Clear suggestion on cursor move
    vim.api.nvim_create_autocmd({'CursorMovedI', 'CursorMoved'}, {
        callback = function()
            clear_suggestion()
        end
    })
end

return M